package br.usp.poli.java.ihc;
/*
Copyright (C) 2008 Fbio Levy Siqueira (fabiolevy@yahoo.com.br)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
import javax.swing.JOptionPane;

import br.usp.poli.java.ICarta;
import br.usp.poli.java.IJogador;

/**
 * Representa um jogador automtico para a interface Homem-Computador.<br>
 * Esse jogador apenas encapsula um jogador normal e repassa as informaes
 * conforme a necessidade.
 * @author FLevy
 */
abstract class JogadorAutomatico extends JogadorIHC {
	protected IJogador jogador;
	
	JogadorAutomatico(IJogador implementacao, String nome, boolean estaEmCima) {
		super(nome, estaEmCima, false);
		this.jogador = implementacao;
	}
	
	public boolean pedirTruco(int valorPartida, ICarta cartaNaMesa) {
		return this.jogador.pedirTruco(valorPartida, cartaNaMesa);
	}

	public boolean analisarTruco(int valorPartida) {
		boolean aceitou = this.jogador.analisarTruco(valorPartida);
		
		if (aceitou) {
			this.valorPartida = valorPartida;
			
			this.getOponente().avisarAceitouTruco(this.valorPartida);
			
			if (getOponente() instanceof JogadorHumano) {
				JOptionPane.showMessageDialog(this.mesa.getPai(),
				    "O computador aceitou o pedido.", 
				    "Resposta",
				    JOptionPane.INFORMATION_MESSAGE);
			}
		} else {
			if (getOponente() instanceof JogadorHumano) {
				JOptionPane.showMessageDialog(this.mesa.getPai(),
				    "O computador recusou o pedido.", 
				    "Resposta",
				    JOptionPane.INFORMATION_MESSAGE);
			}
		}
		
		return aceitou;
	}

	public void avisarGanhouMao(boolean ganhou, int valor) {
		super.avisarGanhouMao(ganhou, valor);
		jogador.avisarGanhouMao(ganhou, valor);
	}

	public void avisarGanhouRodada(boolean ganhou) {
		super.avisarGanhouRodada(ganhou);
		jogador.avisarGanhouRodada(ganhou);
	}

	protected ICarta jogarCarta(ICarta cartaNaMesa) {
		this.mao.podeJogar();
		ICarta carta = jogador.jogar(cartaNaMesa);
		this.mao.jogar(carta);
		return carta;
		
	}
}
