package br.usp.poli.java.ihc;
/*
Copyright (C) 2008 Fbio Levy Siqueira (fabiolevy@yahoo.com.br)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
import br.usp.poli.java.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * Representa uma mo para o Jogo de truco. A mo  composta por
 * 3 cartas, sendo que elas podem ser usadas ou no, ou visveis
 * ou no.<br>
 * A forma mais elegante de se fazer isso era criar duas classes
 * para tratar de cada tipo de mo. Entretanto, por questes de
 * interface, prefiriu-se colocar na mesma classe...
 * @author FLevy
 */
@SuppressWarnings("serial")
class Mao extends JPanel {
	private JComponent[] cartas;
	private ICarta[] cartasOriginais;
	private JLabel nome;
	private JButton pedido;
	private JogadorIHC jogador;
	private boolean nomeAcima, visivel, podeJogar, temCartaJogada;
	private int cartaJogada;
	private boolean pediuTruco;
	
	/**
	 * Cria uma mo com as cartas invisveis.
	 * @param nomeDoJogador O nome do jogador
	 * @param nomeAcima Se o nome do jogador deve aparecer acima ou abaixo
	 * das cartas.
	 */
	Mao(JogadorIHC jogador, boolean nomeAcima, boolean visivel) {
		super(new BorderLayout());
		if (jogador == null)
			throw new IllegalArgumentException("O jogador no pode ser nulo.");
		
		this.setBackground(Truco.obterCor("fundo"));
		
		this.cartas = new JComponent[3];
		this.cartasOriginais = new ICarta[3];
		this.jogador = jogador;
		this.nomeAcima = nomeAcima;
		this.visivel = visivel;
	}
	
	/**
	 * Cria a tela apenas com o espao, mas sem as cartas.
	 */
	void iniciar() {
		int espaco, alturaAreaPedidoTruco;
		try {
			espaco = Integer.parseInt(Truco.obterPropriedade("espaco"));
			alturaAreaPedidoTruco = Integer.parseInt(Truco.obterPropriedade("altura_area_pedido_truco"));
		} catch (NumberFormatException e) {
			throw new ProblemaInterface("Erro ao obter as propriedades do arquivo de configurao: espaco no  inteiro.");
		}
		
		// criando um painel interno para as cartas
		JPanel interno = new JPanel(new FlowLayout());
		interno.setBackground(Truco.obterCor("fundo"));
		this.add(Box.createRigidArea(new Dimension(4*(FiguraDaCarta.largura() + espaco), espaco)), BorderLayout.PAGE_START);
		this.add(Box.createRigidArea(new Dimension(4*(FiguraDaCarta.largura() + espaco), espaco)), BorderLayout.PAGE_END);
		
		// colocando o espao para as cartas
		for (int i = 0; i < 3; i++) {
			if (visivel) {
				cartas[i] = new JButton(FiguraDaCarta.obterTraseira());
				((JButton) cartas[i]).setBackground(Truco.obterCor("fundo"));
				((JButton) cartas[i]).addActionListener(new EventoCarta(i));
				cartas[i].setVisible(false);
				((JButton) cartas[i]).setBorderPainted(false);
			} else {
				cartas[i] = new JLabel(FiguraDaCarta.obterTraseira());
				cartas[i].setVisible(false);
			}
			cartas[i].setPreferredSize(new Dimension(FiguraDaCarta.largura(), FiguraDaCarta.altura()));
			
			interno.add(cartas[i]);
		}
		interno.add(Box.createRigidArea(new Dimension(espaco, FiguraDaCarta.altura())));
		
		this.add(interno, BorderLayout.CENTER);
		
		// colocando o nome
		JPanel painelNome = new JPanel(new FlowLayout());
		painelNome.setBackground(Truco.obterCor("fundo"));
		nome = new JLabel(jogador.getNome());
		nome.setForeground(Truco.obterCor("texto_nome"));
		painelNome.add(nome);
		if (nomeAcima) this.add(painelNome, BorderLayout.PAGE_START);
		else this.add(painelNome, BorderLayout.PAGE_END);
		
		// colocando o boto de truco
		pedido = new JButton();
		resetarValorPedidoTruco();
		
		if (jogador instanceof JogadorHumano) {
			pedido.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					SwingUtilities.invokeLater(new Runnable() {
					    public void run() {
					    	pedirTruco();
					    }
					});
				}
			});
		}
		JPanel painelPedido = new JPanel();
		
		painelPedido.setBackground(Truco.obterCor("fundo"));
		painelPedido.add(pedido);
		painelPedido.add(Box.createRigidArea(new Dimension(1, alturaAreaPedidoTruco)), BorderLayout.LINE_START);
		painelPedido.add(Box.createRigidArea(new Dimension(1, alturaAreaPedidoTruco)), BorderLayout.LINE_END);
		
		pedido.setVisible(false);

		if (nomeAcima) this.add(painelPedido, BorderLayout.PAGE_END);
		else this.add(painelPedido, BorderLayout.PAGE_START);
		
	}
	
	void receberCartas(ICarta[] cartasOriginais) {
		// Guardando as cartas originais
		for (int i = 0; i < cartasOriginais.length; i++) {
			this.cartasOriginais[i] = cartasOriginais[i];
		}
		
		// colocando as cartas
		ImageIcon figura;
		
		for (int i = 0; i < 3; i++) {
			if (visivel) {
				// Cartas do usurio
				figura = FiguraDaCarta.obterCarta(cartasOriginais[i]);
				((JButton) cartas[i]).setIcon(figura);
			}
			cartas[i].setVisible(true);
		}
		podeJogar = false;
		if (visivel) {
			resetarValorPedidoTruco();
			pediuTruco = false;
			this.pedido.setEnabled(false);
			this.pedido.setVisible(true);
		}
	}
	
	void podeJogar() {
		podeJogar = true;
		if (visivel) {
			for (JComponent carta : cartas) {
				((JButton) carta).setBorderPainted(true);
			}
		}
		if (visivel) pedido.setEnabled(true);
	}
	
	void piscarNome() {
		if (this.isEnabled()) {
			if (Truco.obterCor("texto_nome").equals(nome.getForeground()))
				nome.setForeground(Truco.obterCor("texto_nome_piscando"));
			else
				nome.setForeground(Truco.obterCor("texto_nome"));
		}
	}
	
	void resetarPiscar() {
		nome.setForeground(Truco.obterCor("texto_nome"));
	}
	
	void jogar(int posicao) {
		if (podeJogar) {
			podeJogar = false;
			cartas[posicao].setVisible(false);
			cartaJogada = posicao;
			temCartaJogada = true;
			if (visivel) {
				for (JComponent carta : cartas) {
					((JButton) carta).setBorderPainted(false);
				}
				
				pedido.setEnabled(false);
			}
		}
	}
	
	void jogar(ICarta carta) {
		if (podeJogar) {
			for(int i=0; i < cartasOriginais.length; i++) {
				if (carta == cartasOriginais[i])
					jogar(i);
			}
		}
	}
	
	private void pedirTruco() {
		// se no pode jogar ou j trucou, desconsidera
		if (!podeJogar || pediuTruco) return;
		
		// impossibilitando que ele jogue
		podeJogar = false;
		this.desabilitarPedidoTruco();
		pediuTruco = true;
	}
	
	void desabilitarPedidoTruco() {
		this.pedido.setVisible(false);
	}
	
	void habilitarPedidoTruco() {
		this.pedido.setVisible(true);
	}
	
	void mudarValorPedidoTruco(int valor) {
		this.pedido.setText("Pedir " + valor);
	}
	
	void resetarValorPedidoTruco() {
		this.pedido.setText("Pedir truco");
	}
	
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		this.resetarPiscar();
		
		
		for (JComponent c : cartas) c.setEnabled(enabled);
		pedido.setVisible(false);
	}
	
	boolean temCartaJogada() {
		return temCartaJogada;
	}
	
	boolean pediuTruco() {
		return this.pediuTruco;
	}
	
	void zerarPedidoTruco() {
		this.pediuTruco = false;
	}
	
	int cartaJogada() {
		if (temCartaJogada) {
			temCartaJogada = false;
			return cartaJogada;
		}
		
		return -1;
	}
	
	private class EventoCarta implements ActionListener{
		private int posicao;
		public EventoCarta(int posicao) {
			this.posicao = posicao;
		}
		
		public void actionPerformed(ActionEvent e) {
			jogar(posicao);
		}
	}
}
