package br.usp.poli.java.ihc;
/*
Copyright (C) 2008 Fbio Levy Siqueira (fabiolevy@yahoo.com.br)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;

import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import br.usp.poli.java.ICarta;
import br.usp.poli.java.IJogador;

@SuppressWarnings("serial")
class Mesa extends JPanel {	
	private Placar placar;
	private Truco pai;
	private JogadorIHC j1, j2;
	private IJogador vencedor;
	private JogoDeTruco jogo;
	private boolean jogoManilhaFixa;
	private int numeroCartasJogadas;
	private ExecucaoJogo jogoEmExecucao;
	private JLabel vira;
	private JLabel[] cartasJogador1, cartasJogador2, vencedores;

	
	Mesa(Truco pai, JogadorIHC j1, JogadorIHC j2, boolean jogoManilhaFixa, JogoDeTruco jogo) {
		super(new BorderLayout());
		this.setBackground(Truco.obterCor("fundo"));
		
		this.pai = pai;
		this.placar = new Placar();
		this.j1 = j1;
		this.j2 = j2;
		this.jogoManilhaFixa = jogoManilhaFixa;
		this.jogo = jogo;
		this.numeroCartasJogadas = 0;
	}
	
	void iniciar() {
		this.setVisible(false);
		
		this.j1.setMesa(this);
		this.j2.setMesa(this);
			
		this.add(criarPainelVira(), BorderLayout.CENTER);
		
		JPanel inferior = new JPanel(new BorderLayout(0, 0));
		inferior.setBackground(Truco.obterCor("fundo"));
		inferior.add(j2.getMao(), BorderLayout.CENTER);
		inferior.add(placar, BorderLayout.LINE_END);
		this.add(inferior, BorderLayout.PAGE_END);
		
		placar.iniciar(j1, j2);
		
		JPanel superior = new JPanel(new BorderLayout(0, 0));
		superior.setBackground(Truco.obterCor("fundo"));
		superior.add(j1.getMao(), BorderLayout.CENTER);
		superior.add(Box.createRigidArea(placar.getPreferredSize()), BorderLayout.LINE_END);
		this.add(superior, BorderLayout.PAGE_START);
		
		// comecando o jogo
		jogoEmExecucao = new ExecucaoJogo(j1, j2, jogo);
		
		jogoEmExecucao.start();
		this.setVisible(true);
	}
	
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		j1.setEnabled(enabled);
		j2.setEnabled(enabled);
	}
	
	private JPanel criarPainelVira() {
		int espaco, offset, numeroCartas, espacoCartas;

		try {
			espaco = Integer.parseInt(Truco.obterPropriedade("espaco"));
			offset = Integer.parseInt(Truco.obterPropriedade("offset"));
			numeroCartas = Integer.parseInt(Truco.obterPropriedade("numero_cartas_monte"));
			espacoCartas = Integer.parseInt(Truco.obterPropriedade("espaco_cartas_monte"));
		} catch (NumberFormatException e) {
			throw new ProblemaInterface("Erro ao obter as propriedades do arquivo de configurao.");
		}
		
		JPanel meioMesa = new JPanel(new FlowLayout(FlowLayout.LEFT));
		
		meioMesa.setBackground(Truco.obterCor("fundo"));
		
		JPanel painelVira = new JPanel(new FlowLayout());
		painelVira.setBackground(Truco.obterCor("fundo"));
		
		// Espao antes
		painelVira.add(Box.createRigidArea(new Dimension(espaco, FiguraDaCarta.altura())));
		
		// Colocando o monte de cartas
		JLayeredPane painelMonte = new JLayeredPane();
		JLabel cartasDeBaixo;
		
		painelMonte.setPreferredSize(new Dimension((numeroCartas-1)*espacoCartas + FiguraDaCarta.largura(), FiguraDaCarta.altura()));
		
		for (int i = 0; i < numeroCartas; i++) {
			cartasDeBaixo = new JLabel(FiguraDaCarta.obterTraseira());
			cartasDeBaixo.setBounds(espacoCartas*i, 0, FiguraDaCarta.largura(), FiguraDaCarta.altura());
			painelMonte.add(cartasDeBaixo, 0);
		}
		painelVira.add(painelMonte);
		
		// espaco
		painelVira.add(Box.createRigidArea(new Dimension(espaco*2, FiguraDaCarta.altura())));
		
		// colocando o vira - se tiver
		if (!jogoManilhaFixa) {
			vira = new JLabel(FiguraDaCarta.obterTraseira());
			painelVira.add(vira);
			vira.setVisible(false);
			
			// Colocando um espaco depois
			painelVira.add(Box.createRigidArea(new Dimension(espaco*2, FiguraDaCarta.altura())));
		}
		
		meioMesa.add(painelVira);
		
		// colocando as cartas dos jogadores
		cartasJogador1 = new JLabel[3];
		cartasJogador2 = new JLabel[3];
		vencedores = new JLabel[3];
		
		JPanel mao = new JPanel(new GridLayout(1, 3, espaco*3, espaco));
		// GridBagLayout apresenta erros... Preciso usar outro layout!
		
		mao.setBackground(Truco.obterCor("fundo"));

		// Colocando as cartas a serem jogadas
		JLayeredPane cartas;
		JPanel cartaMao;
		for (int i = 0; i < 3; i++) {
			cartas = new JLayeredPane();
			cartas.setPreferredSize(new Dimension(FiguraDaCarta.largura() + offset, FiguraDaCarta.altura() + offset));
			cartasJogador1[i] = new JLabel(FiguraDaCarta.obterTraseira());
			cartasJogador2[i] = new JLabel(FiguraDaCarta.obterTraseira());
			
			cartasJogador1[i].setBounds(0, 0, FiguraDaCarta.largura(), FiguraDaCarta.altura());
			cartas.add(cartasJogador1[i], 1);
			cartasJogador1[i].setVisible(false);
			
			cartasJogador2[i].setBounds(offset, offset, FiguraDaCarta.largura(), FiguraDaCarta.altura());
			cartas.add(cartasJogador2[i], 0);
			cartasJogador2[i].setVisible(false);
			
			cartaMao = new JPanel(new BorderLayout());
			cartaMao.setBackground(Truco.obterCor("fundo"));
			cartaMao.add(cartas, BorderLayout.CENTER);
			
			vencedores[i] = new JLabel(" ", JLabel.CENTER);
			vencedores[i].setForeground(Truco.obterCor("texto_nome_vencedor_mao"));
			cartaMao.add(vencedores[i], BorderLayout.PAGE_END);
			
			mao.add(cartaMao);
		}
		
		meioMesa.add(mao);
		
		return meioMesa;
	}
	
	void jogarCarta(ICarta carta) {
		if (this.numeroCartasJogadas > 6)
			throw new ProblemaInterface("No podem ser jogadas  mesa mais de 6 cartas.");
		
		if (this.numeroCartasJogadas%2 == 0) {
			this.cartasJogador1[numeroCartasJogadas/2].setIcon(FiguraDaCarta.obterCarta(carta));
			this.cartasJogador1[numeroCartasJogadas/2].setVisible(true);
			this.cartasJogador1[numeroCartasJogadas/2].setEnabled(true);
		} else {
			this.cartasJogador2[numeroCartasJogadas/2].setIcon(FiguraDaCarta.obterCarta(carta));
			this.cartasJogador2[numeroCartasJogadas/2].setVisible(true);
			this.cartasJogador2[numeroCartasJogadas/2].setEnabled(true);
		}
		
		numeroCartasJogadas++;
	}
	
	Truco getPai() {
		return this.pai;
	}
	
	void setVira(ICarta carta) {
		if (jogoManilhaFixa) return;
		
		this.vira.setIcon(FiguraDaCarta.obterCarta(carta));
		this.vira.setVisible(true);
	}
	
	void fimRodada(IJogador vencedor) {
		// deixando os labels inacessveis
		this.cartasJogador1[numeroCartasJogadas/2 - 1].setEnabled(false);
		this.cartasJogador2[numeroCartasJogadas/2 - 1].setEnabled(false);
		
		// definindo o vencedor
		this.vencedores[numeroCartasJogadas/2 - 1].setText(vencedor.getNome());
	}
	
	boolean viraJaDefinido() {
		return this.vira.isVisible();
	}
	
	void pontuar(IJogador vencedor, int pontos) {
		this.placar.pontuar(vencedor, pontos);
	}
	
	void fimMao() {
		if (!jogoManilhaFixa) this.vira.setVisible(false);
		this.numeroCartasJogadas = 0;
		for (int i=0; i < 3; i++) {
			this.cartasJogador1[i].setVisible(false);
			this.cartasJogador2[i].setVisible(false);
			this.vencedores[i].setText(" ");
		}
	}
	
	private void fimDoJogo(IJogador jogadorVencedor) {
		this.setEnabled(false);
		this.vencedor = jogadorVencedor;
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				placar.vencedor(vencedor);
				pai.fimDoJogo();
			}
		});
	}
	
	JogadorIHC[] jogadores() {
		JogadorIHC[] vetor = {j1, j2};
		return vetor;
	}
	
	void terminar() {
		this.setEnabled(false);
		jogoEmExecucao.interrupt();
	}
	
	private class ExecucaoJogo extends Thread {
		private JogadorIHC j1, j2;
		private JogoDeTruco jogo;
		
		public ExecucaoJogo(JogadorIHC j1, JogadorIHC j2, JogoDeTruco jogo) {
			this.j1 = j1;
			this.j2 = j2;
			this.jogo = jogo;
		}
		
		public void run() {
			IJogador vencedor = null;
	    	try {
	    		vencedor = jogo.iniciar(j1, j2);
	    	} catch (ExcecaoParada parada) {
	    		// fim!!!!
	    		return;
	    	} catch (Exception e) {
	    		e.printStackTrace();
	    		System.exit(1);
	    	}
	    	fimDoJogo(vencedor);
		}
	}
}