package br.usp.poli.java.ihc;
/*
Copyright (C) 2008 Fbio Levy Siqueira (fabiolevy@yahoo.com.br)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.HashMap;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.JPanel;

import br.usp.poli.java.IJogador;

@SuppressWarnings("serial")
final class Placar extends JPanel {
	private HashMap<JogadorIHC, DadosPlacar> dadosJogadores;
	
	Placar() {
		super(new BorderLayout());
		this.setBackground(Truco.obterCor("fundo"));
		dadosJogadores = new HashMap<JogadorIHC, DadosPlacar>();
	}
	
	void iniciar(JogadorIHC j1, JogadorIHC j2) {
		JLabel placarJogador1, placarJogador2;
		JLabel nome1, nome2;
		
		JPanel interno = new JPanel(new GridBagLayout());
		interno.setBackground(Truco.obterCor("fundo_placar"));
		interno.setBorder(BorderFactory.createEtchedBorder(Truco.obterCor("fundo_placar"), Color.BLACK));
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		
		// Linha 0
		JLabel titulo = new JLabel("PLACAR");
		titulo.setForeground(Truco.obterCor("texto_normal_placar"));
		titulo.setHorizontalAlignment(JLabel.CENTER);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 2;
		c.insets = new Insets(5, 5, 10, 5);
		interno.add(titulo, c);
				
		// Jogador 1
		nome1 = new JLabel(j1.getNome());
		nome1.setForeground(Truco.obterCor("texto_normal_placar"));
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 1;
		c.insets = new Insets(5, 10, 2, 5);
		interno.add(nome1, c);
		
		placarJogador1 = new JLabel("0", JLabel.RIGHT);
		placarJogador1.setPreferredSize(new Dimension(20, 15));
		placarJogador1.setForeground(Truco.obterCor("texto_normal_placar"));
		c.fill = GridBagConstraints.EAST;
		c.anchor = GridBagConstraints.FIRST_LINE_END;
		c.gridx = 1;
		c.gridy = 1;
		c.insets = new Insets(5, 5, 2, 10);
		interno.add(placarJogador1, c);
		
		// Jogador 2
		nome2 = new JLabel(j2.getNome());
		nome2.setForeground(Truco.obterCor("texto_normal_placar"));
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 2;
		c.insets = new Insets(5, 10, 2, 5);
		interno.add(nome2, c);
		
		placarJogador2 = new JLabel("0", JLabel.RIGHT);
		placarJogador2.setPreferredSize(new Dimension(20, 15));
		placarJogador2.setForeground(Truco.obterCor("texto_normal_placar"));
		c.fill = GridBagConstraints.EAST;
		c.anchor = GridBagConstraints.FIRST_LINE_END;
		c.gridx = 1;
		c.gridy = 2;
		c.insets = new Insets(5, 5, 2, 10);
		interno.add(placarJogador2, c);
		
		// Linha 3 ... rea rgida
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 3;
		c.insets = new Insets(5, 5, 2, 10);
		interno.add(Box.createRigidArea(new Dimension(85, 1)), c);
		
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1;
		c.gridy = 3;
		c.insets = new Insets(5, 5, 2, 10);
		interno.add(Box.createRigidArea(new Dimension(10, 1)), c);
		
		JPanel externo = new JPanel(new BorderLayout(4, 4));
		externo.setBackground(Truco.obterCor("fundo"));
		externo.add(interno, BorderLayout.CENTER);
		externo.add(Box.createRigidArea(new Dimension(1, 1)), BorderLayout.PAGE_END);
		externo.add(Box.createRigidArea(new Dimension(1, 1)), BorderLayout.LINE_END);
		
		this.add(externo, BorderLayout.PAGE_END);
		
		// criando os objetos com o placar
		dadosJogadores.put(j1, new DadosPlacar(nome1, placarJogador1, 0));
		dadosJogadores.put(j2, new DadosPlacar(nome2, placarJogador2, 0));
	}
	
	void pontuar(IJogador vencedor, int pontos) {
		DadosPlacar dados = dadosJogadores.get(vencedor);
		if (dados == null)
			throw new ProblemaInterface("Deseja-se pontuar um jogador que no  conhecido.");
		
		dados.pontos += pontos;
		
		if (dados.pontos/10 > 0) {
			dados.labelPontos.setText(dados.pontos + "");
		} else {
			dados.labelPontos.setText(" " + dados.pontos);
		}
		
	}
	
	void vencedor(IJogador vencedor) {
		DadosPlacar dados = dadosJogadores.get(vencedor);
		if (dados == null)
			throw new ProblemaInterface("O vencedor no  um jogador conhecido.");
		
		dados.labelNome.setForeground(Truco.obterCor("texto_vencedor_placar"));
		dados.labelPontos.setForeground(Truco.obterCor("texto_vencedor_placar"));
	}
	
	private class DadosPlacar {
		public JLabel labelNome;
		public JLabel labelPontos;
		public int pontos;
		
		public DadosPlacar(JLabel labelNome, JLabel labelPontos, int pontos) {
			this.labelNome = labelNome;
			this.labelPontos = labelPontos;
			this.pontos = pontos;
		}
	}
}
